<?php

namespace App\Jobs;

use App\BO\BimFileBO;
use App\Libraries\BIM\BIMFactory;
use App\Models\BimFile;
use App\Models\File;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Http\UploadedFile;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Storage;

/**
 * 处理BIM的轻量化转换
 */
class ConvertBimFile implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private $fileId;
    private BimFileBO $bimFileBO;
    private $fileStream;

    /**
     * Create a new job instance.
     */
    public function __construct(int $fileId, BimFileBO $bimFileBO, ?UploadedFile $fileStream = null)
    {
        $this->fileId = $fileId;
        $this->bimFileBO = $bimFileBO;
        $this->fileStream = $fileStream;
    }

    /**
     * Execute the job.
     */
    public function handle(): void
    {
        $file = File::find($this->fileId);

        if ($file && $file->bimFile) {
            if (empty($this->fileStream)) {
                $this->bimFileBO->modelDownloadUrl = Storage::url($file->pathname);
            }

            $bimInfo = BIMFactory::make($file->bimFile->bim_driver)->uploadFile($this->fileStream, $this->bimFileBO);

            BimFile::where('file_id', $file->id)->update($bimInfo);
        }
    }
}
